#include "GameGrid.h"
/*
Constructeur
	Initialise la taille de grille  9
	Initialise son contenu  0 (vide)
	Rsoud la grille pour donner la solution d'une grille vide...
*/
GameGrid::GameGrid()
{
	RG=9;	
	hintX=0;
	hintY=0;
    for(int i=0;i<9;i++)
        for(int j=0;j<9;j++)
			currentGrid[i][j]=initGrid[i][j]=0;		

	solveGrid();
}
/*
Modifieur
	Met  jour la valeur d'une case de la grille et vrifie que la nouvelle est bien dans l'intervalle autorise (0-9)
@param row
	La ligne de la case  mettre  jour
@param col
	La colonne de la case  mettre  jour
@param value
	La nouvelle valeur de la case
*/
void GameGrid::setNewValue(int row, int col,int value)
{
	currentGrid[row][col]=value;
	if(getGridElement(row,col)>9)
		currentGrid[row][col]=0;
	else if(getGridElement(row,col)<0)
		currentGrid[row][col]=9;
}
/*
Efface la grille en cours en l'crasant avec la grille initiale
*/
void GameGrid::clearGrid()
{
	for(int i=0;i<9;i++)
		for(int j=0;j<9;j++)
			currentGrid[i][j]=initGrid[i][j];
}
/*
Accesseur
@param row
	La ligne de la case  retourner
@param col
	La colonne de la case  retourner
@return La valeur de la case dfinie par les coordonnes passes en paramtre (row,col)
*/
int GameGrid::getGridElement(int row, int col)
{
    return currentGrid[row][col];   
}
/*
Teste si une case contient un chiffre fixe (d'origine)
@param row
	La ligne de la case  tester
@param col
	La colonne de la case  tester
@return 
	True si la case est fixe
	False si la case peut tre remplie
*/
bool GameGrid::isFixed(int row, int col)
{
	if(initGrid[row][col])
		return true;
	return false;
}
/*
Charge en mmoire un fichier contenant une grille
@param path
	Le chemin vers la grille  charger
@return 
	True si la grille charge est solvable
	False si la grille charge est insolvable
*/
bool GameGrid::loadGrid(std::string path)
{			
	
	char file2Load[200];
	//conversion string to char* pour le fopen()
	unsigned int i;
	for(i=0;i<path.length();i++)
		file2Load[i]=path.at(i);
	file2Load[i]='\0';

	FILE *f;	
	f=fopen(file2Load,"r");
	char c;
	cleanGrid();
	for(int i=0;i<RG;i++)
		for(int j=0;j<RG;j++)
		{
			c=fgetc(f);
			if(c!='\n')
			{
				if(c=='0')//Si ce sont des @ => 0 => vide
					currentGrid[i][j]=0;			
				else if(c<58 && c>48)//Si ce sont des fixes
					currentGrid[i][j]=initGrid[i][j]=c-'0';		
				else
					currentGrid[i][j]=c-'@';
			}
			else
				j--;
		}
	
	fclose(f);

	name.assign(path);

	if(!checkGrid())//plus rapide que de solver la grille donc gain de temps si la grille ne respecte pas les rgles.
		return false;
	if(!solveGrid())
		return false;
		
	return true;
	
}
/*
Sauvegarde une grille en mmoire dans un fichier
@param path
	Le chemin vers le fichier de sauvegarde
*/
void GameGrid::saveGrid(std::string path)
{
	char file2Save[200];
	//conversion string to char* pour le fopen()
	unsigned int i;
	if(path.empty())//quickSave
	{
		for(i=0;i<name.length();i++)
			file2Save[i]=name.at(i);
		file2Save[i]='\0';
	}
	else//save as...
	{
		for(i=0;i<path.length();i++)
			file2Save[i]=path.at(i);
		file2Save[i]='\0';
	}

	FILE *f;
	f=fopen(file2Save,"w");

	for(int i=0;i<RG;i++)//impression de la grille dans le fichier
	{
		for(int j=0;j<RG;j++)
		{
			if(isFixed(i,j))
				fprintf(f,"%d",getGridElement(i,j));
			else if(getGridElement(i,j))
				fprintf(f,"%c",getGridElement(i,j)+'@');
			else
				fprintf(f,"0");
		}
		fprintf(f,"\n");
	}
	
	fclose(f);
	if(!path.empty())//Mise  jour du nom de la grille
		name.assign(path);	
}
/*
Vrifie si la grille respecte les rgles du jeu
@return 
	True si les rgles sont respectes
	False sinon
*/
bool GameGrid::checkGrid()
{		
	
	for(int i=0;i<RG;i++)
		for(int j=0;j<RG;j++)
		{		
			if(!gridElementIsOk(i,j))			
				return false;
		}
	return true;
}
/*
Teste si la valeur de la case est valable (utiliser dans le vrifieur)
@param row
	La ligne de l'lment  tester
@param col
	La colonne de l'lment  tester
@return 
	True si l'lment respecte les rgles du Sudoku
	False si l'lment entre en contradiction avec une des rgles du Sudoku
*/
bool GameGrid::gridElementIsOk(int row, int col)
{
	int valeur = currentGrid[row][col];
	int k;
	int l;
	int m;
	int RG2=3;
	
	int carre_x = row/RG2;
	int carre_y = col/RG2;
	
	
	if(valeur)
	{		
		for (k = 0; k < RG; k++) 
		{
			if(k!=col && currentGrid[row][k] == valeur)
				return false;
					
			if(k!=row && currentGrid[k][col] == valeur)
				return false;	
		}
		
		for (l = 0; l < RG2; l++) 
			for (m = 0; m < RG2; m++)	
			{
				if(currentGrid[carre_x * RG2 + l][carre_y * RG2 + m] == valeur && (carre_x * RG2 + l)!=row && (carre_y * RG2 + m)!=col)
					return false;
			}			
	}
	return true;
}

/*
Teste si la valeur de la case+1 est valable (utiliser dans le solveur)
@param  row
	La ligne de la case  tester
@param col
	La colonne de la case  tester
@return
	True si le test est Ok
	False sinon
*/
bool GameGrid::testIsOk(int row, int col)
{
	int valeur = solvedGrid[row][col]+1;
	int k;
	int l;
	int m;
	int RG2=3;
	
	int carre_x = row/RG2;
	int carre_y = col/RG2;
	
	for (k = 0; k < RG; k++) 
		if(solvedGrid[row][k] == valeur || solvedGrid[k][col] == valeur)
			return false;
		
	for (l = 0; l < RG2; l++) 
		for (m = 0; m < RG2; m++)
			if(solvedGrid[carre_x * RG2 + l][carre_y * RG2 + m] == valeur)
				return false;
    
	return true;
}
/*
Copie la grille rsolue (au chargement ou  la gnration) dans la grille courante
*/
void GameGrid::solve()
{
	for(int i=0;i<RG;i++)
		for(int j=0;j<RG;j++)
			currentGrid[i][j]=solvedGrid[i][j];
}
/*
Rsoud la grille initiale (initGrid[ ][ ])  et copie la solution en mmoire (solvedGrid[ ][ ])
@return 
	True si la grille a t rsolue
	False si la grille est insolvable
*/
bool GameGrid::solveGrid()
{	
	int i=0;
	int j=0;
	int first_i;
	int first_j;
	
	for(i=0;i<9;i++)
        for(j=0;j<9;j++)
			solvedGrid[i][j]=initGrid[i][j];
   
	i=0;
	j=0;
	/*On cherche la premire case vide*/
	while (initGrid[i][j] != 0) 
	{
		if (j == 8) 
		{
			j = 0;
			i++; 
		}
		else
			j++; 
	}
	first_i=i;
	first_j=j;

	/*On ne s'arrte pas tant que toutes les cases ne sont pas remplies*/
	while (i < RG)
	{
		while (!testIsOk(i,j))/*On incrmente la valeur de la case tant qu'une valeur ne respecte pas les rgles du Sudoku*/
			solvedGrid[i][j]++;
		  
		solvedGrid[i][j]++;/*On incrmente la valeur car dans le test, on teste pour la valeur au dessus*/
		
		/*CAS OU LA GRILLE EST IRRESOLVABLE*/
		if(i == first_i && j == first_j && solvedGrid[i][j]==10) /*si on est  la premire case vide et qu'aucunes valeurs ne convient*/
			return false;//Alors la grille est insolvable !
			
		if (solvedGrid[i][j] == 10)/*Cas o aucune valeur n'est possible*/
		{
			solvedGrid[i][j] = 0;/*On remet  0 la case qui a atteint le max = 10*/
			if(j == 0)/*on revient  la case prcedente*/
			{
				j = 8;
				i--;
			}
			else
				j--;
	  
			while (initGrid[i][j] != 0)/*On recule tant que l'on tombe sur une case contenant un chiffre fixe*/
			{
				if (j == 0) 
				{
					j = 8;
					i--;
				}
				else
					j--;
			}	  
		}
		else if (j == 8)/*Si la valeur est < 10, on continue en passant  la case suivante*/
		{
			j = 0;
			i++;
		} 
		else
			j++;
	  
		while (initGrid[i][j] != 0) /*On cherche la prochaine case vide */
		{
			if (j == 8) 
			{
				j = 0;
				i++;
			} 
			else
				j++;
		}
	}

   return true;
}

/*
Rinitialise la grille  0 = vide
*/
void GameGrid::cleanGrid()
{
	for(int i=0;i<9;i++)
		for(int j=0;j<9;j++)
			currentGrid[i][j]=solvedGrid[i][j]=initGrid[i][j]=0;
}
/*
Gnre une grille alatoire de difficult variante
@param difficulty
	La difficult de la grille : 0  3 = facile  difficile
*/
void GameGrid::randomGrid(int difficulty)
{
	srand(time(NULL));
	int maxItems=40-(difficulty*5);//difficultes :0,1,2,3	Donc de 25  40 cases gnres
	int num;
	int row, col;
	cleanGrid();
	for(int k=0;k<RG;k++)//On remplit alatoirement une dizaine de case
	{
		do
		{
			row=rand()%RG;
			col=rand()%RG;
		}while(isFixed(row,col));
		
		num=(rand()%(RG-1))+1;
		currentGrid[row][col]=num;
		while(!gridElementIsOk(row,col))
		{
			currentGrid[row][col]++;
			if(currentGrid[row][col]>RG)
				currentGrid[row][col]=1;
		}	
		initGrid[row][col]=currentGrid[row][col];
	}
	solveGrid();
	
	for(int i=0;i<RG;i++)
		for(int j=0;j<RG;j++)
			currentGrid[i][j]=initGrid[i][j]=solvedGrid[i][j];
	
	for(int k=RG*RG;k>maxItems;k--)//On fait des trous dans notre grille pleine
	{
		do
		{
			row=rand()%RG;
			col=rand()%RG;
		}while(!currentGrid[row][col]);
		
		initGrid[row][col]=currentGrid[row][col]=0;
	}
	//Efface l'ancien nom
	name.clear();
	
	if(!solveGrid())//Si on tombe sur une grille insolvable (sens ne jamais arriver mais par scurit....), on regnre une nouvelle grille
		randomGrid(difficulty);
	
}
/*
Teste si un lment de la grille est faux, ajoute un lment bonus dans le cas contraire.
@return
	1 si une case est mauvaise
	2 si une case bonus a t ajoute
	3 si la grille est pleine (pas besoin d'indice)	
*/
int GameGrid::fillRandomItem()
{			
	//cherche les mauvaises cases
	for(int i=0;i<RG;i++)
		for(int j=0;j<RG;j++)
			if(currentGrid[i][j]!=solvedGrid[i][j] && currentGrid[i][j])
			{
				hintX=i;
				hintY=j;
				return 1;
			}	
			
	//cherche une case  dcouvrir
	int k,l;
	k=rand()%RG;
	l=rand()%RG;
	
	for(int i=k;i<RG;i++)
		for(int j=l;j<RG;j++)				
			if(!currentGrid[i][j])
			{	
				hintX=i;
				hintY=j;
				currentGrid[i][j]=solvedGrid[i][j];			
				return 2;
			}
	for(int i=RG-1;i>=0;i--)
		for(int j=RG-1;j>=0;j--)
			if(!currentGrid[i][j])
				{	
					hintX=i;
					hintY=j;
					currentGrid[i][j]=solvedGrid[i][j];							
					return 2;
				}
			
	return 3; //Grille pleine
}
/*
Teste si la grille a un nom
@return 
	True si la grille a un nom
	False sinon
*/
bool GameGrid::hasName()
{
	if(name.empty())
		return false;
	return true;
}
/*
Accesseur
@return la ligne de l'indice
*/
int GameGrid::getHintX()
{
	return hintX;
}
/*
Accesseur
@return la colonne de l'indice
*/
int GameGrid::getHintY()
{
	return hintY;
}
/*
Teste si la grille courante est pleine
@return
	True si la grille est pleine
	False sinon
*/
bool GameGrid::isFull()
{
	for(int i=0;i<RG;i++)
		for(int j=0;j<RG;j++)
			if(!currentGrid[i][j])
				return false;
	return true;
}
/*
Accesseur
@return le nom de la grille 
*/
std::string GameGrid::getName()
{
	return name;
}
