#include "MainWindow.h"
/*
Constructeur
	Initialise la fentre principale avec les menus, actions, grille, Zozor, musique, etc
	Dfinit la taille, style, agencement de la fentre
	Dclare les autres objets....
*/
MainWindow::MainWindow()
{
    createActions();
    createMenus();
    createStateBar();                         
    setMinimumSize(610,600);
	setMaximumSize(610,600);
    setWindowIcon(QIcon("images/sudozero.ico"));
    setWindowTitle("SudoZro");
	
	QWidget *centralWidget=new QWidget;
	QGridLayout *GLayout=new QGridLayout;
	QHBoxLayout *HLayout=new QHBoxLayout;
	QVBoxLayout *VLayout=new QVBoxLayout;
	
		
	gridView=new GridView;	
	
	checkButton=new QPushButton("Vrifier");

	checkButton->setIcon(QIcon("images/buttons/check.png"));
	checkButton->setToolTip("Vrifier que la grille rspecte les rgles du jeu");	
	checkButton->setFixedHeight(30);
	connect(checkButton,SIGNAL(clicked()),this,SLOT(checkGrid()));
	
	solveButton=new QPushButton("Rsoudre");

	solveButton->setIcon(QIcon("images/buttons/solve.png"));
	solveButton->setToolTip("Laisser l'ordinateur rsoudre la grille");		
	solveButton->setFixedHeight(30);
	connect(solveButton,SIGNAL(clicked()),this,SLOT(solveGrid()));
	
	clearGridButton=new QPushButton("Rinitialiser");

	clearGridButton->setIcon(QIcon("images/buttons/clear.png"));
	clearGridButton->setToolTip("Vider toutes les cases remplies");	
	clearGridButton->setFixedHeight(30);
	connect(clearGridButton,SIGNAL(clicked()),this,SLOT(clearGrid()));
	
	hintButton=new QPushButton("Indice");

	hintButton->setIcon(QIcon("images/buttons/hint.png"));
	hintButton->setToolTip("Obtenir de l'aide");
	hintButton->setFixedHeight(30);
	connect(hintButton,SIGNAL(clicked()),this,SLOT(giveHint()));
		
	GLayout->addWidget(hintButton,1,1);
	GLayout->addWidget(clearGridButton,1,2);
	GLayout->addWidget(checkButton,2,1);
	GLayout->addWidget(solveButton,2,2);
	
	VLayout->addWidget(gridView);
	VLayout->addLayout(GLayout);
	
	HLayout->addLayout(VLayout);
	
	//theme
	theme=new Theme();
	
	zozor=new QWidget;
	zozor->setFixedSize(QSize(184,480));
	
	help=new QLabel(zozor);
	help->setGeometry(0,0,184,110);
	help->setMargin(5);
	help->setAlignment(Qt::AlignCenter);
	help->setScaledContents(true);
	help->setStyleSheet("background:none;");
	help->setText("Bienvenue dans SudoZro !\n\nConsultez l'aide prsente dans le\nmenu si vous jouez pour la\npremire fois.");
		
	HLayout->addWidget(zozor);
	
	centralWidget->setLayout(HLayout);
	
	setCentralWidget(centralWidget);
	
	//sons
	ambient=new QSound("sounds/ambient.wav");
	ambient->setLoops(-1);
	ambient->play();
	
	
	applyTheme();
		    
}
void MainWindow::createActions()
{
	randomGridAction=new QAction("G&rille alatoire",this);
	randomGridAction->setShortcut(QKeySequence("Ctrl+A"));
	randomGridAction->setIcon(QIcon("images/buttons/random.png"));
	connect(randomGridAction,SIGNAL(triggered()),this,SLOT(randomGrid()));
	
	hintAction=new QAction("Donner un indice",this);
	hintAction->setIcon(QIcon("images/buttons/hint.png"));
	connect(hintAction,SIGNAL(triggered()),this,SLOT(giveHint()));
	
	solveAction=new QAction("Rsoudre la grille",this);
	solveAction->setIcon(QIcon("images/buttons/solve.png"));
	connect(solveAction,SIGNAL(triggered()),this,SLOT(solveGrid()));
	
	checkAction=new QAction("Vrifier la grille", this);
	checkAction->setIcon(QIcon("images/buttons/check.png"));
	connect(checkAction,SIGNAL(triggered()),this,SLOT(checkGrid()));
	
	clearGridAction=new QAction("Effacer la grille",this);
	clearGridAction->setIcon(QIcon("images/buttons/clear.png"));
	connect(clearGridAction,SIGNAL(triggered()),this,SLOT(clearGrid()));
	
    loadGameAction=new QAction("&Charger une grille",this);
	loadGameAction->setShortcut(QKeySequence("Ctrl+C"));
    loadGameAction->setIcon(QIcon("images/buttons/load.png"));	
    connect(loadGameAction,SIGNAL(triggered()),this,SLOT(loadGame()));
	
	saveGameAction=new QAction("Sauver la grille sous...",this);
	saveGameAction->setShortcut(QKeySequence("Ctrl+Z"));
    saveGameAction->setIcon(QIcon("images/buttons/save.png"));
    connect(saveGameAction,SIGNAL(triggered()),this,SLOT(saveGame()));
	
	quickSaveAction=new QAction("&Sauver",this);
	quickSaveAction->setShortcut(QKeySequence("Ctrl+S"));
	quickSaveAction->setIcon(QIcon("images/buttons/quickSave.png"));	
	connect(quickSaveAction,SIGNAL(triggered()),this,SLOT(quickSave()));
            
    quitGameAction=new QAction("&Quitter", this);
	quitGameAction->setShortcut(QKeySequence("Ctrl+Q"));
    quitGameAction->setIcon(QIcon("images/buttons/quit.png"));	
    connect(quitGameAction,SIGNAL(triggered()),qApp,SLOT(quit()));
	
	//musique
	playMusicAction=new QAction("&Lecture",this);
	playMusicAction->setIcon(QIcon("images/buttons/playMusic.png"));	
	connect(playMusicAction,SIGNAL(triggered()),this,SLOT(playMusic()));
	
	stopMusicAction=new QAction("Stop",this);
	stopMusicAction->setIcon(QIcon("images/buttons/stopMusic.png"));
	connect(stopMusicAction,SIGNAL(triggered()),this,SLOT(stopMusic()));
	
	//themes
	selectThemeAction=new QAction("Slectionner un thme", this);
	selectThemeAction->setIcon(QIcon("images/buttons/selectTheme.png"));
	connect(selectThemeAction,SIGNAL(triggered()),this,SLOT(selectTheme()));
	
	//aide
	helpAction=new QAction("Aide", this);
	connect(helpAction,SIGNAL(triggered()),this,SLOT(openHelp()));
}
/*
Instancie et configure les menus
*/
void MainWindow::createMenus()
{
    QMenu *fileMenu=menuBar()->addMenu("&Fichier");
	QMenu *gridMenu=menuBar()->addMenu("&Grille");
	QMenu *soundMenu=menuBar()->addMenu("&Musique");
	QMenu *themeMenu=menuBar()->addMenu("&Thme");
	QMenu *helpMenu=menuBar()->addMenu("?");
	
    fileMenu->addAction(loadGameAction);
	fileMenu->addAction(saveGameAction);
	fileMenu->addAction(quickSaveAction);
	fileMenu->addSeparator();
    fileMenu->addAction(quitGameAction);
	
	gridMenu->addAction(randomGridAction);
    gridMenu->addAction(hintAction);
	gridMenu->addAction(checkAction);
	gridMenu->addAction(solveAction);
	gridMenu->addAction(clearGridAction);
	
	soundMenu->addAction(playMusicAction);
	soundMenu->addAction(stopMusicAction);
	
	themeMenu->addAction(selectThemeAction);
	
	helpMenu->addAction(helpAction);
	
	toolBar=addToolBar("Outils");	
	toolBar->addAction(loadGameAction);
	toolBar->addAction(saveGameAction);
	toolBar->addAction(quickSaveAction);
	toolBar->addSeparator();
	toolBar->addAction(randomGridAction);
	toolBar->addAction(hintAction);
	toolBar->addAction(checkAction);
	toolBar->addAction(solveAction);
	toolBar->addAction(clearGridAction);
	toolBar->addSeparator();
	toolBar->addAction(playMusicAction);
	toolBar->addAction(stopMusicAction);
	toolBar->addSeparator();
	toolBar->addAction(selectThemeAction);
	toolBar->setMovable(false);
}
/*
Instancie et configure la barre d'tat
*/
void MainWindow::createStateBar()
{
    stateBar=statusBar();
	
	timerLabel=new QLabel;
	timerLabel->setFixedSize(QSize(40,20));
		
	timer=new Chrono;
	connect(timer,SIGNAL(timeout()),this,SLOT(updateTimerLabel()));
	
	stateBar->addPermanentWidget(timerLabel,2);
}
/*
Applique le thme  la fentre
*/
void MainWindow::applyTheme()
{
	zozor->setStyleSheet(QString("background-image:url(").append(theme->getZozorPath()).append(");background-repeat:no-repeat;"));
	gridView->setNewGridBackground(theme->getGridPath());
}
/*
Dmarre le chrono
*/
void MainWindow::startTimer()
{
	timer->reset();
	timer->start();
	timerLabel->clear();
	
}
/*
Stoppe le chrono
*/
void MainWindow::stopTimer()
{
	timer->stop();
}


//SLOTS
/*
Rsoud la grille et affiche le dnouement
Stoppe le timer1
*/
void MainWindow::solveGrid()
{
	gridView->getGameGrid()->solve();
	stateBar->showMessage("La grille a t rsolue avec succs");
	help->setText("Votre grille est rsolue.\n\nRinitialisez la grille si vous\nsouhaitez rejouer sur la mme grille.");
	gridView->refreshGridView();	
	gridView->setEnabled(false);
	
	stopTimer();
}    
/*
Vrifie que la grille respecte les rgles du Sudoku
*/
void MainWindow::checkGrid()
{
	if(gridView->getGameGrid()->checkGrid())
	{
		stateBar->showMessage("La grille respecte les rgles du Sudoku");
		if(gridView->getGameGrid()->isFull())
		{
			help->setText("Flicitation !\n\nLa grille est correcte !");
			stopTimer();
		}
		else
			help->setText("La grille actuelle respecte les\nrgles du Sudoku.");
		
	}
	else
	{
		stateBar->showMessage("La grille ne respecte pas les rgles du Sudoku !");
		help->setText("La grille ne respecte pas les rgles\ndu Sudoku.\n\nLe contenu d'une des cases rentre\nen contradiction avec une autre.");
	}
}
/*
Demande le niveau de difficult de la grille  gnrer
Lance la gnration de la grille
*/
void MainWindow::randomGrid()
{
	QString gridName("Grille alatoire | Niveau : ");
	QStringList difficulties;
    difficulties << "Dbutant" << "Habitu" << "Expert" << "Vtran";
	
	QString choixNiveau;
	bool ok;
	choixNiveau=QInputDialog::getItem(this, "Difficult de la grille", "Quel est votre niveau ?", difficulties,0,false,&ok);
	
	if(ok && !difficulties.isEmpty())
	{
		int niveau=0;
		if(!choixNiveau.compare(QString("Dbutant")))
		{
			niveau=0;
			gridName.append("Dbutant");
		}
		else if(!choixNiveau.compare(QString("Habitu")))
		{
			niveau=1;
			gridName.append("Habitu");
		}
		else if(!choixNiveau.compare(QString("Expert")))
		{
			niveau=2;
			gridName.append("Expert");
		}
		else if(!choixNiveau.compare(QString("Vtran")))
		{
			niveau=3;
			gridName.append("Vtran");
		}
			
		gridView->getGameGrid()->randomGrid(niveau);
		gridView->newButtonGrid();
		gridView->refreshGridView();
		stateBar->showMessage("Grille alatoire gnre");
		help->setText("A vous de jouer !\n\nClic gauche : incrmenter la case\nClic droit : dcrmenter la case\nClic central : effacer la case");		
	}
	refreshTitle(gridName);
	startTimer();
	
}
/*
Demande l'emplacement de la grille  charger
Charge la grille
Avertit si la grille est insolvable
*/
void MainWindow::loadGame()
{
	QString fileToLoad = QFileDialog::getOpenFileName(this, "Ouvrir une grille", "grids", "Grilles (*.gri)");
	if(!fileToLoad.isEmpty())
	{		
		if(gridView->getGameGrid()->loadGrid(fileToLoad.toStdString()))//Si la grille charge est solvable
		{
			gridView->newButtonGrid();
			gridView->refreshGridView();
			stateBar->showMessage(fileToLoad);
			help->setText("A vous de jouer !\n\nClic gauche : incrmenter la case\nClic droit : dcrmenter la case\nClic central : effacer la case");
		}
		else
		{
			stateBar->showMessage("La grille que vous souhaitez charger est insolvable !");
			help->setText("La grille que vous souhaitez\ncharger est insolvable.\n\nChoisissez en une autre.");
		}
	}
	refreshTitle(QString::fromStdString(gridView->getGameGrid()->getName()));
	startTimer();
	
}
/*
Demande l'emplacement de sauvegarde de la grille
Lance la sauvegarde
*/
void MainWindow::saveGame()
{
	QString fileToSave = QFileDialog::getSaveFileName(this, "Enregistrer une grille","grids", "Grilles (*.gri)");
	if(!fileToSave.isEmpty())
	{		
		gridView->getGameGrid()->saveGrid(fileToSave.toStdString());
		stateBar->showMessage(fileToSave);
		help->setText("La grille a bien t sauvegarde.");
	}
	refreshTitle(QString::fromStdString(gridView->getGameGrid()->getName()));	
}
/*
Effectue une sauvegarde rapide si l'emplacement du fichier est connu
Redirige vers MainWindow::saveGame( ) sinon
*/
void MainWindow::quickSave()
{
	if(!gridView->getGameGrid()->hasName())
		saveGame();
	else
	{
		gridView->getGameGrid()->saveGrid("");		
		stateBar->showMessage("La sauvegarde rapide a t effectue",3000);
		help->setText("La sauvegarde rapide a eu lieu.");
	}
}
/*
Lance la rinitialisation de la grille
*/
void MainWindow::clearGrid()
{
	gridView->clearGrid();
	gridView->refreshGridView();
	gridView->setEnabled(true);
	stateBar->showMessage("La grille a t rinitialise");
	help->setText("La grille a t rinitialise pour\nque vous puissiez repartir de zro.");
	
	startTimer();
}
/*
Lance la procdure demandant un indice
Affiche  l'cran le rsultat
*/
void MainWindow::giveHint()
{
	int result=gridView->getGameGrid()->fillRandomItem();
	
	if(result==1)//case mal remplie
	{		
		QString message("La case (ligne : ");
		message.append(QString().setNum(gridView->getGameGrid()->getHintX()+1));
		message.append(", colonne : ");
		message.append(QString().setNum(gridView->getGameGrid()->getHintY()+1));
		message.append(") est mal remplie");
		stateBar->showMessage(message);
		help->setText("Une case a t mal remplie.\n\nSon chiffre est indiqu en rouge.");
		gridView->refreshGridView();
		gridView->showWrongItem();
	}
	else if(result==2)
	{
		QString message("La case (ligne : ");
		message.append(QString().setNum(gridView->getGameGrid()->getHintX()+1));
		message.append(", colonne : ");
		message.append(QString().setNum(gridView->getGameGrid()->getHintY()+1));
		message.append(") a t remplie pour vous aider  continuer");
		stateBar->showMessage(message);
		help->setText("Une case vient d'tre correctement\nremplie pour vous aider.\n\nSon chiffre est indiqu en vert.");
		gridView->refreshGridView();
		gridView->showItemAdded();
	}
	else if(result==3)
	{
		stateBar->showMessage("La grille est pleine, vous n'avez pas besoin d'indice !");
		help->setText("La grille est pleine.\n\nCliquez sur le bouton de\nvrification pour savoir\nsi votre grille est correcte.");
	}
	
}
/*
Lance la musique
*/
void MainWindow::playMusic()
{
	ambient->play();
	stateBar->showMessage("La musique d'ambiance a t dmarre");
	help->setText("Vous pouvez arrter la musique\nd'ambiance en cliquant sur la\nnote rouge.");
}
/*
Stoppe la musique
*/
void MainWindow::stopMusic()
{
	ambient->stop();
	stateBar->showMessage("La musique d'ambiance a t stoppe");
	help->setText("Vous pouvez redmarrer la musique\nd'ambiance en cliquant sur la\nnote verte.");	
}
/*
Demande le thme  utiliser
*/
void MainWindow::selectTheme()
{
	QString grid;
	QString zozor;
	
	QDir *zozorDir=new QDir("themes/zozors");
	
	QStringList zozors=zozorDir->entryList();
	
	QStringList themeList;
	
	for(int i=0;i<zozors.size();i++)
	{
		if(zozors.at(i)!="." && zozors.at(i)!="..")//On exclut le rpertoire courant et parent de la liste
		{
			QString tmp=zozors.at(i);
			tmp.chop(4);//pour enlever l'extension des images
			
			themeList<<tmp;
		}		
	}
	bool ok;
	QString choixTheme=QInputDialog::getItem(this, "Thme du jeu", "Choisissez un thme :", themeList,0,false,&ok);
	
	if(ok && !choixTheme.isEmpty())
	{		
		theme->setNewTheme(choixTheme);
		help->setText("J'aime mon nouveau costume !");
		applyTheme();
	}
}
/*
Met  jour l'affichage du chrono
*/
void MainWindow::updateTimerLabel()
{	
	timer->updateTimer();
	QString time;
	time.append(QString::number(timer->getHour()));
	time.append(":");
	time.append(QString::number(timer->getMin()));
	time.append(":");
	time.append(QString::number(timer->getSec()));
	
	timerLabel->setText(time);
	
}
/*
Met  jour le titre de la fentre en concatnant le titre du jeu et le nom de la grille en cours
*/
void MainWindow::refreshTitle(QString newTitle)
{
	QString title;
	QStringList tmpList=newTitle.split("/");
	title.append("SudoZro - ");
	title.append(tmpList.takeLast());
	setWindowTitle(title);
}
/*
Ouvre la fentre d'aide
*/
void MainWindow::openHelp()
{
	new HelpWindow(this);
}
