#ifndef HEADER_MAINWINDOW
#define HEADER_MAINWINDOW
#include <QtGui>

#include "Chrono.h"
#include "Theme.h"
#include "GridView.h"
#include "HelpWindow.h"
/*
Gre la fentre principale de l'application
*/
class MainWindow : public QMainWindow
{
    Q_OBJECT
    private:
		QAction *randomGridAction;//Action pour gnrer alatoirement une grille
		QAction *hintAction;//Action pour demander un indice
		QAction *solveAction;//Action pour rsoudre la grille
		QAction *checkAction;//Action pour vrifier la grille
		QAction *clearGridAction;//Action pour vider la grille
		QAction *loadGameAction;//Action pour charger une grille
		QAction *saveGameAction;//Action pour sauver une grille
		QAction *quickSaveAction;//Action pour sauver rapidement une grille
        QAction *quitGameAction;//Action pour quitter le jeu
		QAction *playMusicAction;//Action pour jouer la musique
		QAction *stopMusicAction;//Action pour stopper la musique
		QAction *selectThemeAction;//Action pour slectionner un thme
		QAction *helpAction;//Action pour afficher l'aide
		
		GridView *gridView;//Vue de la grille (Voir classe GridView)
		
		QPushButton *checkButton;//Bouton pour vrifier la grille
		QPushButton *solveButton;//Bouton pour rsoudre la grille
		QPushButton *clearGridButton;//Bouton pour vider la grille
		QPushButton *hintButton;//Bouton pour demander un indice
		
		QToolBar *toolBar;//Barre d'outils
		QStatusBar *stateBar;//Barre d'tat
		QSound *ambient;//Musique d'ambiance
		
		QLabel *help;//Message d'aide de la bulle de Zozor
		QWidget *zozor;//Zozor
		
		Theme *theme;//Thme du jeu
		
		Chrono *timer;//Chronomtre du jeu
		QLabel *timerLabel;//Afficheur du chrono
						
    public:
        MainWindow();
		
    private:
        void createActions();
        void createMenus();
        void createStateBar();	
		void applyTheme();  
		void startTimer();
		void stopTimer();
		void refreshTitle(QString newTitle);
				
    private slots:
		void randomGrid();
        void loadGame();
		void saveGame();
		void quickSave();
        void checkGrid();
        void solveGrid();   
		void clearGrid();
		void giveHint();
		void playMusic();
		void stopMusic();
		void selectTheme();
		void updateTimerLabel();
		void openHelp();
		
};
#endif
